/* (C) 2005 by Marcin Wiacek www.mwiacek.com */

#ifndef gsm_cal_h
#define gsm_cal_h

#ifdef WIN32
#  include <windows.h>
#endif

#include "../gsmcomon.h"
#include "../misc/misc.h"

typedef enum {
	Calendar_Not_Assigned = 1,

	Calendar_Text_Text,
	Calendar_Text_Location,
	Calendar_Text_Phone,

	Calendar_DateTime_Start,
	Calendar_DateTime_End,
	Calendar_DateTime_End_Repeat,
	Calendar_DateTime_ToneAlarm,
	Calendar_DateTime_SilentAlarm,

	Calendar_Int_Repeat_Frequency,
	Calendar_Int_Repeat_DayOfWeek,
	Calendar_Int_Repeat_Day,
	Calendar_Int_Repeat_Month
} GSM_Calendar_SubEntryType;

typedef enum {
	Calendar_Type_Not_Assigned = 1,

	Calendar_Type_Meeting,
	Calendar_Type_Memo,
	Calendar_Type_Call,
	Calendar_Type_Birthday,
	Calendar_Type_Reminder
} GSM_Calendar_Type;

class GSM_CalendarSubEntry
{
	friend class GSM_CalendarEntry;
public:
	GSM_CalendarSubEntry();
	~GSM_CalendarSubEntry();

	GSM_Calendar_SubEntryType 	GetType();
	GSM_Error 			SetToText(GSM_Calendar_SubEntryType Typ, wchar_t *Txt);
	wchar_t				*GetText();
	GSM_Error 			SetToDateTime(GSM_Calendar_SubEntryType Typ, GSM_DateTime DT2);
	GSM_DateTime			*GetDateTime();
	GSM_Error 			SetToInt(GSM_Calendar_SubEntryType Typ, int Value);
	int				GetInt();

	GSM_CalendarSubEntry		*GetNext();
private:
	void 				SetNext(GSM_CalendarSubEntry *Nxt);

	int				IntValue;
	GSM_DateTime			DT;
	GSM_CalendarSubEntry		*Next;
	wchar_t				*Text;
	GSM_Calendar_SubEntryType	Type;
};

class GSM_CalendarEntry
{
public:
	GSM_CalendarEntry();
	~GSM_CalendarEntry();

	GSM_Calendar_Type		Type;
	int				Location;

	void 				ClearAll();	
	BOOLEAN 			GetNext(GSM_CalendarSubEntry **En);
	GSM_Error	 		AddText(GSM_Calendar_SubEntryType Typ, wchar_t *Txt);
	GSM_Error	 		AddDateTime(GSM_Calendar_SubEntryType Typ, GSM_DateTime DT2);
	GSM_Error	 		AddInt(GSM_Calendar_SubEntryType Typ, int Value);
private:
	GSM_CalendarSubEntry		*Entries;
};

#endif
